/**
*****************************************************************************
* File: ALFAT.c												   
* Copyright 2012 GHI Electronics, LLC 								   
* his is a very basic driver to use ALFAT SoC Processor
* For demonstration purposes only. not to be used in production.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.								   *
************************************************************************
**/

#include "ALFAT.h"


UINT8 dataRx[DATA_RX_SIZE];
UINT8 commands[CMD_BUFFER_SIZE];
void ALFAT_Reset(void)
{
		// TODO
		// Pull ALFAT reset pin to low
	  // Delay few ms
	  // Release ALFAT reset pin
	  // Delay at least 50us
	 
}

/////// Encode error code ///////////
/* This funtion will encdoe "!00\n"*/
////////////////////////////////////
INT32 EncodeErrorCode(UINT8 *data1, char *data2)
{
	if (strcmp((char*)&data1,data2)!=0)
		{
			return ERR_UNEXPECTED;
		}
		return ERR_NO_ERROR;
}

void ALFAT_ClearRespond()
{	
	while(IsDataReady())
	{
		ReadBytes(dataRx, 0,1);	
	}
}
void ResetRxBuffer()
{
		memset(dataRx, 0, DATA_RX_SIZE);
}
INT32 ALFAT_ReadBanner()
{
		int i=0;
		while(IsDataReady())
		{
				i+=ReadBytes(dataRx, i,1);
		}
		return EncodeErrorCode(dataRx,"\n GHI Electronics, LLC \n----------------------\n ALFAT SoC Processor \n!00\n");
}
/////// Get Version ///////////
// Send :"V\r"               //
// Return "vx.x.x\n!00\n     //
///////////////////////////////
INT32 ALFAT_GetVersion()
{	
		int i=0;
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
	
		AddCmd("V");	
		SendCmd();
	
		while(IsDataReady()== FALSE); // wait for data ready
		memset(dataRx,0	,DATA_RX_SIZE);		
		while(IsDataReady()== TRUE) 
		{
			i+=ReadBytes(dataRx, i,1);
		}
		return EncodeErrorCode(dataRx, "v2.0.0\n!00\n");
	
}
////// Initialize a Media drive //////////
// commands: I<SP>X:<LF>                 //
// Return : !00<LF>						//
// X: Media drive such as U0, U1, M     //
//////////////////////////////////////////
INT32 ALFAT_InitializeMedia(UINT8 drv[])
{
		int i = 0;
	
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
	
		AddCmd("I ");
		AddCmd(drv);
		SendCmd();
	
		while(IsDataReady()== FALSE); // wait for data ready
		
		memset(dataRx,0	,DATA_RX_SIZE);	
		while(IsDataReady())
		{
			i+=ReadBytes(dataRx, i,1);
		}
		return EncodeErrorCode(dataRx,"!00\n");
}		

////// Open Handle ////////////////
// commands: O<SP>nM>full path<LF>//
// Return:  !00<LF>              //
/////////////////////////////////// 
INT32 ALFAT_OpenFile(UINT8 handle,UINT8* filename, UINT8 *mode)
{
		UINT32 i=0;
		UINT8 handle_hex[8];
		// Conert handle from INT to HEX string 
		ConvertIntToHex(handle,handle_hex);
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
	
		AddCmd("O ");
		AddCmd(handle_hex);
		AddCmd(mode);
		AddCmd(">");
		AddCmd(filename);
		SendCmd();
		// Wait for data ready
		while(IsDataReady()== FALSE); // wait for data ready
		memset(dataRx,0	,DATA_RX_SIZE);	
		// Now has something to read
		while( IsDataReady())
		{	
			i +=ReadBytes(dataRx, i,1);
		}
		return EncodeErrorCode(dataRx,"!00\n");

}
////// Write to a file  /////////////
// commands: W<SP>n>ssssssss<LF>    //
// Or     : L<SP>n>ssssssss<LF>    //
// Return 1: !00<LF>               //
// User sends data (ssssssss bytes)//
// Return 2: $aaaaaaaa<LF>!00<LF>  //
/////////////////////////////////////
INT32 ALFAT_WriteFile(UINT8 handle,UINT8* data, INT32 length)
{
		UINT8 handle_hex[8];
		UINT8 length_hex[8];		
		INT32 i =0;
		
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
		
		ConvertIntToHex(handle,handle_hex);
		ConvertIntToHex(length,length_hex);	
		AddCmd( "W "); 
		AddCmd(handle_hex);
		AddCmd(">");	 
		AddCmd(	length_hex);
		SendCmd();
	
		memset(dataRx,0,DATA_RX_SIZE);
		while(IsDataReady()== FALSE); // wait for !00\n
		// Get first "!00\n" if ALFAT accept this commands	
		i = 0;
	  while( IsDataReady() && i <4)
		{
				i+=ReadBytes(dataRx, i,1);
		}		
		if( EncodeErrorCode(dataRx,"!00\n")!= ERR_NO_ERROR)
		{
				return ERR_UNEXPECTED;
		}
		// Write data
		WriteBytes(data,0, length);	
		// Wait for $aaaaaaaa\n!00\n
		while(IsDataReady()== FALSE); 
		// Now get $aaaaaaaa\n!00\n
		ResetRxBuffer();
		i = 0;
		while( IsDataReady() && i<14)
		{	
			i +=ReadBytes(dataRx, i,1);
		}
		// There will be 14 bytes, we will check last 4 bytes for Error code	
		return EncodeErrorCode((UINT8 *)&dataRx[10],"!00\n");

}
////// Write to a file  /////////////
// commands: L<SP>n>ssssssss<LF>    //
// Or     : L<SP>n>ssssssss<LF>    //
// Return 1: !00<LF>               //
// User sends data (ssssssss bytes)//
// Return 2: $aaaaaaaa<LF>!00<LF>  //
/////////////////////////////////////
INT32 ALFAT_WriteFileDMA(UINT8 handle,UINT8* data, INT32 length)
{
		UINT8 handle_hex[8];
		UINT8 length_hex[8];		
		INT32 i =0;
		
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
		
		ConvertIntToHex(handle,handle_hex);
		ConvertIntToHex(length,length_hex);	
		AddCmd( "L "); 
		AddCmd(handle_hex);
		AddCmd(">");	 
		AddCmd(	length_hex);
		SendCmd();
	
		memset(dataRx,0,DATA_RX_SIZE);
		// wait for data ready
		while(IsDataReady()== FALSE); 
		// Get first "!00\n" if ALFAT accept this commands			
		i = 0;
	  while( IsDataReady() && i <4)
		{
				i+=ReadBytes(dataRx, i,1);
		}		
		if( EncodeErrorCode(dataRx,"!00\n")!= ERR_NO_ERROR)
		{
				return ERR_UNEXPECTED;
		}	
		//Wait for BUSY pin go to low
		while(IsBusy()== TRUE); 
		//Now Busy pin is low, ALFAT is ready to receive with DMA
		WriteBytesDMA(data, 0, length);
		// Wait for $aaaaaaaa\n!00\n
		while(IsDataReady()== FALSE); 
		// Now get $aaaaaaaa\n!00\n
		ResetRxBuffer();
		i = 0;
		while( IsDataReady() && i<14)
		{	
			i +=ReadBytes(dataRx, i,1);
		}
		// There will be 14 bytes, we will check last 4 bytes for Error code	
		return EncodeErrorCode((UINT8 *)&dataRx[10],"!00\n");
	
}
///// Read data from File /////////////////////////////////////////////
// Command: R<SP>nM>ssssssss<LF>                                     //
// Return : !00<LF>ssssssss Bytes are returned $aaaaaaaa<LF>!00<LF>  //
///////////////////////////////////////////////////////////////////////								
INT32 ALFAT_ReadFile(INT32 handle,UINT8 *buffer,INT32 length, UINT8 *filler)
{
		UINT8 handle_hex[8] ;
		UINT8 length_hex[8] ;
		int i;
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();

		ConvertIntToHex(handle,handle_hex);
		ConvertIntToHex(length,length_hex);

		AddCmd( "R ");
		AddCmd(handle_hex);
		AddCmd(filler);
		AddCmd(">");
		AddCmd(length_hex);
		SendCmd();
	
		memset(dataRx,0,DATA_RX_SIZE);
		// wait data ready
		while(IsDataReady()== FALSE); 
		// Get first "!00\n" if ALFAT accept this commands	
		i = 0;
	  while( IsDataReady() && i <4)
		{
				i+=ReadBytes(dataRx, i,1);
		}		
		if( EncodeErrorCode(dataRx,"!00\n")!= ERR_NO_ERROR)
		{
				return ERR_UNEXPECTED;
		}
		// Read raw data
		ResetRxBuffer();
		i = 0;	
		while( IsDataReady() && i< length)
		{	
			i +=ReadBytes(dataRx, i,1); 			
		}
		// Wait for $aaaaaaaa\n!00\n
		while(IsDataReady()== FALSE); 
		// Now get $aaaaaaaa\n!00\n
		ResetRxBuffer();
		i = 0;
		while( IsDataReady() && i<14)
		{	
			i +=ReadBytes(dataRx, i,1);
		}
		// There will be 14 bytes, we will check last 4 bytes for Error code	
		return EncodeErrorCode((UINT8 *)&dataRx[10],"!00\n");

}
///// Close a Handle //////////
// Command: C<SP>n<LF>       //
// Return : !00<LF>          //
///////////////////////////////
INT32 ALFAT_CloseFile(INT32 handle)
{
		UINT8 handle_hex[8];
		int i=0;
		ALFAT_ClearRespond();
		ResetRxBuffer();
		ClearCmd();
		
		ConvertIntToHex(handle,handle_hex);
		AddCmd( "C ");
		AddCmd( handle_hex);
		SendCmd();
		// wait data ready
		while(IsDataReady()== FALSE); 
		// Now has something to read
		while( IsDataReady())
		{	
			i +=ReadBytes(dataRx, i,1);
		}
		return EncodeErrorCode(dataRx,"!00\n");
}


//////////////////////////////////////////////// 
//////// Covert a interger to hex string ///////
////////////////////////////////////////////////
void ConvertIntToHex( UINT32 number, UINT8* number_str)
{
	UINT8 hex[16] = "0123456789ABCDEF";
	INT32 i=0;
	INT32 j=0;
	INT32 shift=0;
	UINT8 c=0;
	UINT32 index = 0;
	UINT8 start =0;
	memset(number_str,0,8);
	number_str[j]='0';
	for (i=0; i<7; i++)
	{
		 index = (UINT32)(((UINT32)((UINT32)(number) & ((UINT32)(0xF000000>>shift)))) >> ((UINT32)(24-shift)));
		 c =  hex[index];
		 if (c!='0') start++;
		 if (c!='0' || start!=0)
		 {
		 	number_str[j]=c;
			j++;
		 }
		 shift+=4;
	}
}

//////////////////////////////////////////////// 
////////  Clear cmd buffer ////////////////////
////////////////////////////////////////////////

void ClearCmd()
{
	memset(commands,0,CMD_BUFFER_SIZE);
}
//////////////////////////////////////////////// 
////////  Add param to cmd ////////////////////
////////////////////////////////////////////////
void AddCmd( UINT8 *s)
{
	strcat((char*)commands,(char*)s);
}
//////////////////////////////////////////////// 
//////// Send cmd to ALFAT  ////////////////////
////////////////////////////////////////////////
void SendCmd()
{
	AddCmd( "\n");
	WriteBytes(commands, 0,strlen((char*)commands));	
	DelayMilisecond(100);
}












