/**
*****************************************************************************
* File: ALFAT.h											   
* Copyright 2012 GHI Electronics, LLC 								   
* his is a very basic driver to use ALFAT SoC Processor
* For demonstration purposes only. not to be used in production.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.								   *
************************************************************************
**/

#include <stdio.h>
#include <ctype.h>                    /* character functions                  */
#include <string.h>                   /* string and memory functions          */

#define TRUE    1
#define FALSE   0

#define INT8 		char 
#define UINT8 	unsigned char 
#define INT16 	short 
#define UINT16 	unsigned short 
#define INT32 	int	
#define UINT32 	unsigned int 
#define INT64 	long 
#define UINT64 	unsigned long 

#define ERR_UNEXPECTED 99
#define ERR_NO_ERROR 0

#define DATA_RX_SIZE 	 1*1024
#define DATA_TX_SIZE   1*1024
#define CMD_BUFFER_SIZE 256
#define USER_TIME_OUT 1000

#define DelayMilisecond(x)
#define DelayMicrosecond(x)

#define SPI_BUS
#ifdef SPI_BUS // using SPI example	
	#include "ALFAT_SPI.h"
	#define WriteBytes   					STM32_SPI_WriteBytes
	#define WriteBytesDMA   			STM32_SPI_WriteBytesDMA
	#define ReadBytes   					STM32_SPI_ReadBytes
	#define OpenBus 							STM32_SPI_Open
	#define IsDataReady	 					STM32_SPI_ReadyPinState
	#define IsBusy								STM32_SPI_BusyPinState			
	#define GetC									STM32_SPI_GetC
	#define PutC									STM32_SPI_PutC	
#endif
#ifdef USART_BUS // using UART example
	#include "ALFAT_UART.h"
	#define WriteBytes   					STM32_USART_WriteBytes
	#define WriteBytesDMA   			STM32_USART_WriteBytesDMA
	#define ReadBytes   					STM32_USART_ReadBytes
	#define OpenBus 							STM32_USART_Open
	#define IsDataReady 					STM32_USART_ReadyPinState
	#define IsBusy								STM32_USART_BusyPinState
	#define GetC									STM32_USART_GetC
	#define PutC									STM32_USART_PutC	
#endif
#ifdef I2C_BUS // using UART example
	#include "ALFAT_I2C.h"
	#define WriteBytes   					STM32_I2C_WriteBytes
	#define ReadBytes   					STM32_I2C_ReadBytes
	#define OpenBus 							STM32_I2C_Open
	#define IsDataReady 					STM32_I2C_ReadyPinState
	#define IsBusy								STM32_I2C_BusyPinState
#endif


#if !defined(SPI_BUS) && !defined(USART_BUS) && !defined(I2C_BUS)
#error You have to define UART/SPI/I2C
#endif
void ALFAT_Reset(void);
void ALFAT_OpenBus(void);
INT32 ALFAT_ReadBanner(void);
INT32 ALFAT_GetVersion(void);
INT32 ALFAT_InitializeMedia(UINT8 *drv);
INT32 ALFAT_CloseFile(INT32 handle);
INT32 ALFAT_WriteFile(UINT8 handle,UINT8* content, INT32 length);
INT32 ALFAT_OpenFile(UINT8 handle,UINT8* filename, UINT8 *mode);
INT32 ALFAT_SendCommand(UINT8 *cmd);
INT32 ALFAT_ReadFile(INT32 handle,UINT8 *buffer,INT32 length, UINT8 *filler);
INT32 EncodeErrorCode(UINT8 *data1, char *data2);
void ResetRxBuffer(void);
void ALFAT_ClearRespond(void);
void ClearCmd(void);
void AddCmd( UINT8 *s);
void SendCmd(void);
void ConvertIntToHex( UINT32 number, UINT8* number_str);







