/**
*****************************************************************************
* File: ALFAT_SPI.c												   
* Copyright 2012 GHI Electronics, LLC 								   
* These are basic configurations to to access ALFAT SoC Processor through SPI interface.
* For demonstration purposes only. not to be used in production.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.								   *
************************************************************************
**/

#include "ALFAT.h"
#ifdef SPI_BUS // using SPI_BUS example
UINT8 spiDataTx[SPI_DATA_TX_BUFFER_SIZE];
INT32 spi_tx_fifo_count;
INT32 spi_tx_fifo_in ;
INT32 spi_tx_fifo_out ; 
UINT8 spiDataRx[SPI_DATA_RX_BUFFER_SIZE];
INT32 spi_rx_fifo_count;
INT32 spi_rx_fifo_in ;
INT32 spi_rx_fifo_out ;
UINT8 spi_rx_data;
UINT8 spi_frame_header_rx[3];
UINT8 spi_frame_header_tx[3];
SPI_InitTypeDef  SPI_InitStructure;



void SPIx_IRQHANDLER(void)
{
		UINT8 c;
		if (SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_RXNE) == SET)
		{
			c = SPI_I2S_ReceiveData(SPIx);
			spiDataRx[spi_rx_fifo_in++] = c;
			spi_rx_fifo_count++;
			if (spi_rx_fifo_in>=SPI_DATA_RX_BUFFER_SIZE)
			{
				spi_rx_fifo_in =0;
			}
		}
		if (SPI_I2S_GetITStatus(SPIx, SPI_I2S_IT_TXE) == SET)
		{

			SPI_I2S_SendData(SPIx, spiDataTx[spi_tx_fifo_out++]);
			if ( spi_tx_fifo_out >=	 SPI_DATA_TX_BUFFER_SIZE)
			{
				spi_tx_fifo_out=0;
			}
			if (spi_tx_fifo_count>0)
			spi_tx_fifo_count--;
			if ( spi_tx_fifo_count==0 )
			{
				SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, DISABLE);
			}

		}
}
void SPI_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Enable the SPI clock */
  SPIx_CLK_INIT(SPIx_CLK, ENABLE);

  /* Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(SPIx_NSS_GPIO_CLK| SPIx_SCK_GPIO_CLK | SPIx_MISO_GPIO_CLK | SPIx_MOSI_GPIO_CLK, ENABLE);

  /* Connect SPI pins to AF5 */  
  GPIO_PinAFConfig(SPIx_SCK_GPIO_PORT, SPIx_SCK_SOURCE, SPIx_SCK_AF);
  GPIO_PinAFConfig(SPIx_MOSI_GPIO_PORT, SPIx_MOSI_SOURCE, SPIx_MOSI_AF);
  GPIO_PinAFConfig(SPIx_MISO_GPIO_PORT, SPIx_MISO_SOURCE, SPIx_MISO_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_DOWN;

  /* SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = SPIx_SCK_PIN;
  GPIO_Init(SPIx_SCK_GPIO_PORT, &GPIO_InitStructure);

  /* SPI  MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPIx_MOSI_PIN;
  GPIO_Init(SPIx_MOSI_GPIO_PORT, &GPIO_InitStructure);


  GPIO_InitStructure.GPIO_Pin =  SPIx_MISO_PIN;
  GPIO_Init(SPIx_MISO_GPIO_PORT, &GPIO_InitStructure);


    /* SPI NSS pin configuration */
  //GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin = SPIx_NSS_PIN;
  //GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_Init(SPIx_NSS_GPIO_PORT, &GPIO_InitStructure);
 
  /* SPI configuration -------------------------------------------------------*/
  SPI_I2S_DeInit(SPIx);
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;
  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  
  /* Configure the Priority Group to 1 bit */                
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
  
  /* Configure the SPI interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = SPIx_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}
/////////////////// Open SPI Bus /////////////////
void STM32_SPI_Open()
{
	SPI_Config();
	SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
	SPI_Init(SPIx, &SPI_InitStructure);
	SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_RXNE , ENABLE);
	SPI_Cmd(SPIx, ENABLE);
	PinSetInput(GPIO_SPI_READY_PORT,GPIO_SPI_READY_PIN,PULL_DOWN);
	PinSetInput(GPIO_SPI_BUSY_PORT,GPIO_SPI_BUSY_PIN,PULL_DOWN);
	spi_tx_fifo_count = 0; 
	spi_rx_fifo_count = 0;
	spi_rx_fifo_in =0;
	spi_rx_fifo_out =0 ;
	spi_tx_fifo_in =0;
	spi_tx_fifo_out =0 ;
	spi_rx_data=0;
}
void  STM32_SPI_PutC(UINT8 b)
{
		while(STM32_SPI_BusyPinState()); // don't send if Busy pin is high
		while (spi_tx_fifo_count>=(SPI_DATA_TX_BUFFER_SIZE));	
		spiDataTx[spi_tx_fifo_in++] = b;
		if (spi_tx_fifo_in>=SPI_DATA_TX_BUFFER_SIZE)
		{
			spi_tx_fifo_in =0;
		}
		__disable_irq();
		spi_tx_fifo_count++;
		SPI_I2S_ITConfig(SPIx, SPI_I2S_IT_TXE, ENABLE);
		__enable_irq();
	
}
UINT8 STM32_SPI_GetC()
{
		while (spi_rx_fifo_out==spi_rx_fifo_in);
		spi_rx_data = spiDataRx[spi_rx_fifo_out++];
		if (spi_rx_fifo_out>= SPI_DATA_RX_BUFFER_SIZE)
		{
				spi_rx_fifo_out =0;
		}
		if (spi_rx_fifo_out==spi_rx_fifo_in) 
		{

				spi_rx_fifo_count =0;
		}
		return spi_rx_data;
}
///////////// SPI write bytes ///////////////////////////
//This function send data to ALFAT directly by SPIx->DR register
UINT32 STM32_SPI_WriteBytesDMA(UINT8 *data, UINT32 index, UINT32 length)
{
	INT32 i =0;		
	for(i=index; i<length;i++)
	{		
		STM32_SPI_PutC(data[i]);
	}
	return (i);	
}

UINT32 STM32_SPI_WriteBytes(UINT8 *data, UINT32 index, UINT32 length)
{
	 INT32 start =index ;
	 INT32 end = length + index;
	 INT32 timeout =	USER_TIME_OUT;
	
		// Add WRITE_CMD (1) to header
		spi_frame_header_tx[0] =  WRITE_CMD;
		//  Add size LSB
		spi_frame_header_tx[1] =  length & 0xFF;
		//  Add size MSB
		spi_frame_header_tx[2] = (length>>8) & 0xFF; 
		spi_frame_header_rx[1] = 0;
		// Send frame for write
		// Add timeout
		while ( spi_frame_header_rx[1] != TRUE && timeout>0)
		{				
				// Send 2 bytes header
				STM32_SPI_PutC(spi_frame_header_tx[0]);
				STM32_SPI_PutC(spi_frame_header_tx[1]);
				// Read 2 bytes back
				spi_frame_header_rx[0] = STM32_SPI_GetC();
				spi_frame_header_rx[1] = STM32_SPI_GetC();
				if (spi_frame_header_rx[1] != TRUE)
				{
						DelayMilisecond(1);
						timeout--;
				}
		}
		// timeout is OK
		if ( timeout!=0) 
		{
				// Send the 3th byte
				STM32_SPI_PutC(spi_frame_header_tx[2]); 
				// Get back 1 byte
				STM32_SPI_GetC();
				while( start<	end)
				{							
						// Send data
						STM32_SPI_PutC(data[start++]);
						// Read data back
						STM32_SPI_GetC();
						DelayMicrosecond(4);
				}
		}
		
	 return (start-index);
}
//////////// SPI Read bytes ////////////////////////
UINT32 STM32_SPI_ReadBytes(UINT8 *data, UINT32 index, UINT32 length)
{
		INT32 actual_size=0;
		INT32 start =index ;
		INT32 end;
		// Prepare frame header
		// Add READ_CMD (2) to header
		spi_frame_header_tx[0] =  READ_CMD;
		//  Add size LSB
		spi_frame_header_tx[1] =  length & 0xFF;
		//  Add size MSB
		spi_frame_header_tx[2] = (length>>8) & 0xFF;
		// Send 3 bytes Header
		STM32_SPI_PutC(spi_frame_header_tx[0]);
		STM32_SPI_PutC(spi_frame_header_tx[1]);
		STM32_SPI_PutC(spi_frame_header_tx[2]);
		// Read  3 bytes header back
		spi_frame_header_rx[0] = STM32_SPI_GetC();
		spi_frame_header_rx[1] = STM32_SPI_GetC();
		spi_frame_header_rx[2] = STM32_SPI_GetC();
		// Get actual size
		actual_size = 	(spi_frame_header_rx[1] | (spi_frame_header_rx[2]<<8));
		// Check actual size
		if (length > actual_size)
		{
				length =  actual_size;
		}
		// Update end
		end = 	length + index;
		while(start<end)
		{
				// Send 0x00 
				STM32_SPI_PutC(0);
				// Read back data
				data[start++] = STM32_SPI_GetC();
		}
		return (start-index);
}
////////////// STM32_SPI_DataReadyPinState /////////////////
// Read status GPIO data ready pin.
// Return high is data ready, low is not data ready
// Re-define this function depend on system
/////////////////////////////////////////////////////////
UINT32 STM32_SPI_ReadyPinState()
{
	return PinStatus(GPIO_SPI_READY_PORT,GPIO_SPI_READY_PIN);	
}
////////////// STM32_SPI_BusyPinState /////////////////
// Read status GPIO busy pin.
// Return high is busy, low is not busy
// Re-define this function depend on system
/////////////////////////////////////////////////////////
UINT32 STM32_SPI_BusyPinState()
{
	return PinStatus(GPIO_SPI_BUSY_PORT,GPIO_SPI_BUSY_PIN);	
}
#endif //SPI_BUS





