/**
*****************************************************************************
* File: ALFAT_UART.c												   
* Copyright 2012 GHI Electronics, LLC 								   
* These are basic configurations to to access ALFAT SoC Processor through USART interface.
* For demonstration purposes only. not to be used in production.
*
* THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
* ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.								   *
************************************************************************
**/
/* Includes ------------------------------------------------------------------*/

#include "ALFAT.h"
#ifdef USART_BUS // using UART example
USART_InitTypeDef USART_InitStructure;
UINT8 uartDataTx[UART_DATA_TX_BUFFER_SIZE];
INT32 uart_tx_fifo_count; 
INT32 uart_tx_fifo_in ;
INT32 uart_tx_fifo_out ;
UINT8 uartDataRx[UART_DATA_RX_BUFFER_SIZE];
INT32 uart_rx_fifo_count;
INT32 uart_rx_fifo_in ;
INT32 uart_rx_fifo_out ;


 
void STM32_USART_Init(USART_InitTypeDef* USART_InitStruct)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Enable GPIO clock */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

  /* Connect PXx to USARTx_Tx*/
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_USART1);

  /* Connect PXx to USARTx_Rx*/
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource10,GPIO_AF_USART1);

  /* Configure USART Tx as alternate function  */
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure USART Rx as alternate function  */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* USART configuration */
  USART_Init(USART1, USART_InitStruct);
    
  /* Enable USART */
  USART_Cmd(USART1, ENABLE);
}
void USART1_IRQHandler(void)
{
  if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET )
  {
	  uartDataRx[uart_rx_fifo_in++] = USART1->DR;//USART->DR;
	  uart_rx_fifo_count++;
	  if (uart_rx_fifo_in>=UART_DATA_RX_BUFFER_SIZE)
	  {
	  	 uart_rx_fifo_in =0;
	  }
  }
  if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET)
  {   
		USART1->DR = uartDataTx[uart_tx_fifo_out++];
		if ( uart_tx_fifo_out >=	 UART_DATA_TX_BUFFER_SIZE)
		{
			uart_tx_fifo_out=0;
		}
		if (uart_tx_fifo_count>0)
		uart_tx_fifo_count--;
		if ( uart_tx_fifo_count==0 )
  		{
	        USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
  		}
  }
}
void NVIC_Config(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}
void Config_UART(INT32 Baudrate)
{
  USART_InitStructure.USART_BaudRate = Baudrate;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

}
void STM32_USART_Open()
{
  Config_UART( 115200);
  STM32_USART_Init(&USART_InitStructure);
  NVIC_Config();
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
  PinSetInput(GPIO_USART_READY_PORT,GPIO_USART_READY_PIN,PULL_DOWN);
  PinSetInput(GPIO_USART_BUSY_PORT,GPIO_USART_BUSY_PIN,PULL_DOWN);
  uart_tx_fifo_count =0;
  uart_tx_fifo_in =0;
  uart_tx_fifo_out =0;
  uart_rx_fifo_count =0;
  uart_rx_fifo_in =0;
  uart_rx_fifo_out =0;
}
void STM32_USART_PutC(UINT8 b)
{
	while (uart_tx_fifo_count>=(UART_DATA_TX_BUFFER_SIZE));	//Bit 7 TXE: Transmit data register empty
	uartDataTx[uart_tx_fifo_in++] = b;
	if (uart_tx_fifo_in>=UART_DATA_TX_BUFFER_SIZE)
	{
		uart_tx_fifo_in =0;
	}
	__disable_irq();
	uart_tx_fifo_count++;
	USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
	__enable_irq();
}
UINT8 STM32_USART_GetC()
{
	UINT8 data;
	while (uart_rx_fifo_out==uart_rx_fifo_in);
	data = uartDataRx[uart_rx_fifo_out++];
	if (uart_rx_fifo_out>= UART_DATA_RX_BUFFER_SIZE)
	{
		uart_rx_fifo_out =0;
	}
	if (uart_rx_fifo_out==uart_rx_fifo_in) 
	{
		uart_rx_fifo_count =0;
	}
	return data;
}
UINT32 STM32_USART_WriteBytes(UINT8 *data, UINT32 index, UINT32 length)
{
	UINT32 start = 	index;
	UINT32 end =  length + index;
	// Send data over USART
	while (start < end)
	{
		// Wait while ALFAT busy
		while(STM32_USART_BusyPinState());
		STM32_USART_PutC(data[start++]);
	}
	return (start-index) ;
}
UINT32 STM32_USART_WriteBytesDMA(UINT8 *data, UINT32 index, UINT32 length)
{
	//None support DMA if UART
	return FALSE;
}
UINT32 STM32_USART_ReadBytes(UINT8 *data, UINT32 index, UINT32 length)
{
	UINT32 start = 	index;
	UINT32 end ;
	UINT32 actual_size = uart_rx_fifo_count;
	// Read actual size on bus
	if (length>	actual_size)
	{
		length = actual_size;
	}
	// Update array
	end =  length +  index;
	// Save data to user buffer
	while  (start<end)
	{

		data[start++] = STM32_USART_GetC();
	}
	return (start-index) ;
}
////////////// STM32_USART_DataReadyPinState /////////////////
// Read status GPIO data ready pin.
// Return high is data ready, low is not data ready
// Re-define this function depend on system
/////////////////////////////////////////////////////////
UINT32 STM32_USART_ReadyPinState()
{
	return PinStatus(GPIO_USART_READY_PORT,GPIO_USART_READY_PIN);	
}
////////////// STM32_USART_BusyPinState /////////////////
// Read status GPIO busy pin.
// Return high is busy, low is not busy
// User maybe re-define this function depend on system
/////////////////////////////////////////////////////////
UINT32 STM32_USART_BusyPinState()
{
	return PinStatus(GPIO_USART_BUSY_PORT,GPIO_USART_BUSY_PIN);	
}
#endif //UART_BUS

